/*****************************************************************************

  ui_font.c

  Original version by NJ (NMAME)

*****************************************************************************/

#include <windows.h>
#include "driver.h"
#include "artwork.h"
#include "ui_pal.h"

#define COLOR_BUTTONS	(85 + 1)

static UINT16 _uifonttable[65536];
static UINT32 colortable[MAX_COLORTABLE];
static int need_font_warning;
static unsigned char *fontdata_db;

/****************************************************************************/

/* in artwork.c */
static int gamescale;

/****************************************************************************/

struct ui_lang_info_t ui_lang_info[UI_LANG_MAX] =
{
	{  "en_US", "US", "English (US)",         1252, 0     },
};

static int current_lang = UI_LANG_EN_US;

int lang_find_langname(const char *name)
{
	int i;

	for (i = 0; i < UI_LANG_MAX; i++)
	{
		if (!stricmp(ui_lang_info[i].name, name))
			return i;
		if (!stricmp(ui_lang_info[i].shortname, name))
			return i;
	}

	return -1;
}

int lang_find_codepage(int cp)
{
	int i;

	for (i = 0; i < UI_LANG_MAX; i++)
		if (ui_lang_info[i].codepage == cp)
			return i;

	return -1;
}

void set_langcode(int langcode)
{
	current_lang = langcode;
}

/*-------------------------------------------------
	ui_rot2raw_rect - convert a rect from rotated
	coordinates to raw coordinates
-------------------------------------------------*/

static void ui_rot2raw_rect(struct rectangle *rect)
{
	int temp, w, h;

	/* get the effective screen size, including artwork */
	artwork_get_screensize(&w, &h);

	/* apply X/Y swap first */
	if (Machine->ui_orientation & ORIENTATION_SWAP_XY)
	{
		temp = rect->min_x; rect->min_x = rect->min_y; rect->min_y = temp;
		temp = rect->max_x; rect->max_x = rect->max_y; rect->max_y = temp;
	}

	/* apply X flip */
	if (Machine->ui_orientation & ORIENTATION_FLIP_X)
	{
		temp = w - rect->min_x - 1;
		rect->min_x = w - rect->max_x - 1;
		rect->max_x = temp;
	}

	/* apply Y flip */
	if (Machine->ui_orientation & ORIENTATION_FLIP_Y)
	{
		temp = h - rect->min_y - 1;
		rect->min_y = h - rect->max_y - 1;
		rect->max_y = temp;
	}
}

/*-------------------------------------------------
	ui_markdirty - mark a raw rectangle dirty
-------------------------------------------------*/

INLINE void ui_markdirty(const struct rectangle *rect)
{
	extern UINT8 ui_dirty;

	artwork_mark_ui_dirty(rect->min_x, rect->min_y, rect->max_x, rect->max_y);
	ui_dirty = 5;
}



static unsigned char* load_fontdata(void)
{
	FILE *fd;
	int fontsize = ui_lang_info[options.langcode].numchars * 24;
	unsigned char *fontdata;

	need_font_warning = 0;

	if (!fontsize)
		return NULL;

	if ((fd = fopen("font\\fontname.fnt", "rb")) == NULL)
	{
		need_font_warning = 1;
		return NULL;
	}

	fontdata = malloc(fontsize);
	memset(fontdata,0,fontsize);
	fread(fontdata, fontsize, 1, fd);
	return fontdata;
}


int uifont_need_font_warning(void)
{
	return need_font_warning;
}


static UINT16 uifont_buildtable(UINT16 code)
{
	UINT8 c1, c2;
	c1 = (code >> 8) & 0xff;
	c2 = code & 0xff;
	
	switch (options.langcode)
	{
		default:
			if(c1 == 0x90 && c2 > 0x80 && c2 < 0x80 + COLOR_BUTTONS)
				code = c2 - 0x80;
			else
				return 0x00;
	}

	return(code);
}

int uifont_buildfont(void)
{
	static UINT8 fontdata6x8[] =
	{
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,	/* [ 0- 1] */
		0x7c,0x80,0x98,0x90,0x80,0xbc,0x80,0x7c,0xf8,0x04,0x64,0x44,0x04,0xf4,0x04,0xf8,	/* [ 2- 3] tape pos 1 */
		0x7c,0x80,0x98,0x88,0x80,0xbc,0x80,0x7c,0xf8,0x04,0x64,0x24,0x04,0xf4,0x04,0xf8,	/* [ 4- 5] tape pos 2 */
		0x7c,0x80,0x88,0x98,0x80,0xbc,0x80,0x7c,0xf8,0x04,0x24,0x64,0x04,0xf4,0x04,0xf8,	/* [ 6- 7] tape pos 3 */
		0x7c,0x80,0x90,0x98,0x80,0xbc,0x80,0x7c,0xf8,0x04,0x44,0x64,0x04,0xf4,0x04,0xf8,	/* [ 8- 9] tape pos 3 */
		0x30,0x48,0x84,0xb4,0xb4,0x84,0x48,0x30,0x30,0x48,0x84,0x84,0x84,0x84,0x48,0x30,	/* [10-11] */
		0x00,0xfc,0x84,0x8c,0xd4,0xa4,0xfc,0x00,0x00,0xfc,0x84,0x84,0x84,0x84,0xfc,0x00,	/* [12-13] */
		0x00,0x38,0x7c,0x7c,0x7c,0x38,0x00,0x00,0x00,0x30,0x68,0x78,0x78,0x30,0x00,0x00,	/* [14-15] circle & bullet */
		0x80,0xc0,0xe0,0xf0,0xe0,0xc0,0x80,0x00,0x04,0x0c,0x1c,0x3c,0x1c,0x0c,0x04,0x00,	/* [16-17] R/L triangle */
		0x20,0x70,0xf8,0x20,0x20,0xf8,0x70,0x20,0x48,0x48,0x48,0x48,0x48,0x00,0x48,0x00,
		0x00,0x00,0x30,0x68,0x78,0x30,0x00,0x00,0x00,0x30,0x68,0x78,0x78,0x30,0x00,0x00,
		0x70,0xd8,0xe8,0xe8,0xf8,0xf8,0x70,0x00,0x1c,0x7c,0x74,0x44,0x44,0x4c,0xcc,0xc0,
		0x20,0x70,0xf8,0x70,0x70,0x70,0x70,0x00,0x70,0x70,0x70,0x70,0xf8,0x70,0x20,0x00,
		0x00,0x10,0xf8,0xfc,0xf8,0x10,0x00,0x00,0x00,0x20,0x7c,0xfc,0x7c,0x20,0x00,0x00,
		0xb0,0x54,0xb8,0xb8,0x54,0xb0,0x00,0x00,0x00,0x28,0x6c,0xfc,0x6c,0x28,0x00,0x00,
		0x00,0x30,0x30,0x78,0x78,0xfc,0x00,0x00,0xfc,0x78,0x78,0x30,0x30,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x20,0x20,0x20,0x20,0x00,0x20,0x00,
		0x50,0x50,0x50,0x00,0x00,0x00,0x00,0x00,0x00,0x50,0xf8,0x50,0xf8,0x50,0x00,0x00,
		0x20,0x70,0xc0,0x70,0x18,0xf0,0x20,0x00,0x40,0xa4,0x48,0x10,0x20,0x48,0x94,0x08,
		0x60,0x90,0xa0,0x40,0xa8,0x90,0x68,0x00,0x10,0x20,0x40,0x00,0x00,0x00,0x00,0x00,
		0x20,0x40,0x40,0x40,0x40,0x40,0x20,0x00,0x10,0x08,0x08,0x08,0x08,0x08,0x10,0x00,
		0x20,0xa8,0x70,0xf8,0x70,0xa8,0x20,0x00,0x00,0x20,0x20,0xf8,0x20,0x20,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x30,0x30,0x60,0x00,0x00,0x00,0xf8,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x30,0x30,0x00,0x00,0x08,0x10,0x20,0x40,0x80,0x00,0x00,
		0x70,0x88,0x88,0x88,0x88,0x88,0x70,0x00,0x10,0x30,0x10,0x10,0x10,0x10,0x10,0x00,
		0x70,0x88,0x08,0x10,0x20,0x40,0xf8,0x00,0x70,0x88,0x08,0x30,0x08,0x88,0x70,0x00,
		0x10,0x30,0x50,0x90,0xf8,0x10,0x10,0x00,0xf8,0x80,0xf0,0x08,0x08,0x88,0x70,0x00,
		0x70,0x80,0xf0,0x88,0x88,0x88,0x70,0x00,0xf8,0x08,0x08,0x10,0x20,0x20,0x20,0x00,
		0x70,0x88,0x88,0x70,0x88,0x88,0x70,0x00,0x70,0x88,0x88,0x88,0x78,0x08,0x70,0x00,
		0x00,0x00,0x30,0x30,0x00,0x30,0x30,0x00,0x00,0x00,0x30,0x30,0x00,0x30,0x30,0x60,
		0x10,0x20,0x40,0x80,0x40,0x20,0x10,0x00,0x00,0x00,0xf8,0x00,0xf8,0x00,0x00,0x00,
		0x40,0x20,0x10,0x08,0x10,0x20,0x40,0x00,0x70,0x88,0x08,0x10,0x20,0x00,0x20,0x00,
		0x30,0x48,0x94,0xa4,0xa4,0x94,0x48,0x30,0x70,0x88,0x88,0xf8,0x88,0x88,0x88,0x00,
		0xf0,0x88,0x88,0xf0,0x88,0x88,0xf0,0x00,0x70,0x88,0x80,0x80,0x80,0x88,0x70,0x00,
		0xf0,0x88,0x88,0x88,0x88,0x88,0xf0,0x00,0xf8,0x80,0x80,0xf0,0x80,0x80,0xf8,0x00,
		0xf8,0x80,0x80,0xf0,0x80,0x80,0x80,0x00,0x70,0x88,0x80,0x98,0x88,0x88,0x70,0x00,
		0x88,0x88,0x88,0xf8,0x88,0x88,0x88,0x00,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x00,
		0x08,0x08,0x08,0x08,0x88,0x88,0x70,0x00,0x88,0x90,0xa0,0xc0,0xa0,0x90,0x88,0x00,
		0x80,0x80,0x80,0x80,0x80,0x80,0xf8,0x00,0x88,0xd8,0xa8,0x88,0x88,0x88,0x88,0x00,
		0x88,0xc8,0xa8,0x98,0x88,0x88,0x88,0x00,0x70,0x88,0x88,0x88,0x88,0x88,0x70,0x00,
		0xf0,0x88,0x88,0xf0,0x80,0x80,0x80,0x00,0x70,0x88,0x88,0x88,0x88,0x88,0x70,0x08,
		0xf0,0x88,0x88,0xf0,0x88,0x88,0x88,0x00,0x70,0x88,0x80,0x70,0x08,0x88,0x70,0x00,
		0xf8,0x20,0x20,0x20,0x20,0x20,0x20,0x00,0x88,0x88,0x88,0x88,0x88,0x88,0x70,0x00,
		0x88,0x88,0x88,0x88,0x88,0x50,0x20,0x00,0x88,0x88,0x88,0x88,0xa8,0xd8,0x88,0x00,
		0x88,0x50,0x20,0x20,0x20,0x50,0x88,0x00,0x88,0x88,0x88,0x50,0x20,0x20,0x20,0x00,
		0xf8,0x08,0x10,0x20,0x40,0x80,0xf8,0x00,0x30,0x20,0x20,0x20,0x20,0x20,0x30,0x00,
		0x40,0x40,0x20,0x20,0x10,0x10,0x08,0x08,0x30,0x10,0x10,0x10,0x10,0x10,0x30,0x00,
		0x20,0x50,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfc,
		0x40,0x20,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x70,0x08,0x78,0x88,0x78,0x00,
		0x80,0x80,0xf0,0x88,0x88,0x88,0xf0,0x00,0x00,0x00,0x70,0x88,0x80,0x80,0x78,0x00,
		0x08,0x08,0x78,0x88,0x88,0x88,0x78,0x00,0x00,0x00,0x70,0x88,0xf8,0x80,0x78,0x00,
		0x18,0x20,0x70,0x20,0x20,0x20,0x20,0x00,0x00,0x00,0x78,0x88,0x88,0x78,0x08,0x70,
		0x80,0x80,0xf0,0x88,0x88,0x88,0x88,0x00,0x20,0x00,0x20,0x20,0x20,0x20,0x20,0x00,
		0x20,0x00,0x20,0x20,0x20,0x20,0x20,0xc0,0x80,0x80,0x90,0xa0,0xe0,0x90,0x88,0x00,
		0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x00,0x00,0x00,0xf0,0xa8,0xa8,0xa8,0xa8,0x00,
		0x00,0x00,0xb0,0xc8,0x88,0x88,0x88,0x00,0x00,0x00,0x70,0x88,0x88,0x88,0x70,0x00,
		0x00,0x00,0xf0,0x88,0x88,0xf0,0x80,0x80,0x00,0x00,0x78,0x88,0x88,0x78,0x08,0x08,
		0x00,0x00,0xb0,0xc8,0x80,0x80,0x80,0x00,0x00,0x00,0x78,0x80,0x70,0x08,0xf0,0x00,
		0x20,0x20,0x70,0x20,0x20,0x20,0x18,0x00,0x00,0x00,0x88,0x88,0x88,0x98,0x68,0x00,
		0x00,0x00,0x88,0x88,0x88,0x50,0x20,0x00,0x00,0x00,0xa8,0xa8,0xa8,0xa8,0x50,0x00,
		0x00,0x00,0x88,0x50,0x20,0x50,0x88,0x00,0x00,0x00,0x88,0x88,0x88,0x78,0x08,0x70,
		0x00,0x00,0xf8,0x10,0x20,0x40,0xf8,0x00,0x08,0x10,0x10,0x20,0x10,0x10,0x08,0x00,
		0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x40,0x20,0x20,0x10,0x20,0x20,0x40,0x00,
		0x00,0x68,0xb0,0x00,0x00,0x00,0x00,0x00,0x20,0x50,0x20,0x50,0xa8,0x50,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x20,0x20,0x40,0x0c,0x10,0x38,0x10,0x20,0x20,0xc0,0x00,
		0x00,0x00,0x00,0x00,0x00,0x28,0x28,0x50,0x00,0x00,0x00,0x00,0x00,0x00,0xa8,0x00,
		0x70,0xa8,0xf8,0x20,0x20,0x20,0x20,0x00,0x70,0xa8,0xf8,0x20,0x20,0xf8,0xa8,0x70,
		0x20,0x50,0x88,0x00,0x00,0x00,0x00,0x00,0x44,0xa8,0x50,0x20,0x68,0xd4,0x28,0x00,
		0x88,0x70,0x88,0x60,0x30,0x88,0x70,0x00,0x00,0x10,0x20,0x40,0x20,0x10,0x00,0x00,
		0x78,0xa0,0xa0,0xb0,0xa0,0xa0,0x78,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x20,0x20,0x00,0x00,0x00,0x00,0x00,
		0x10,0x10,0x20,0x00,0x00,0x00,0x00,0x00,0x28,0x50,0x50,0x00,0x00,0x00,0x00,0x00,
		0x28,0x28,0x50,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x78,0x78,0x30,0x00,0x00,
		0x00,0x00,0x00,0x78,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfc,0x00,0x00,0x00,0x00,
		0x68,0xb0,0x00,0x00,0x00,0x00,0x00,0x00,0xf4,0x5c,0x54,0x54,0x00,0x00,0x00,0x00,
		0x88,0x70,0x78,0x80,0x70,0x08,0xf0,0x00,0x00,0x40,0x20,0x10,0x20,0x40,0x00,0x00,
		0x00,0x00,0x70,0xa8,0xb8,0xa0,0x78,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x50,0x88,0x88,0x50,0x20,0x20,0x20,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x00,0x20,0x20,0x20,0x20,0x20,0x00,
		0x00,0x20,0x70,0xa8,0xa0,0xa8,0x70,0x20,0x30,0x48,0x40,0xe0,0x40,0x48,0xf0,0x00,
		0x00,0x48,0x30,0x48,0x48,0x30,0x48,0x00,0x88,0x88,0x50,0xf8,0x20,0xf8,0x20,0x00,
		0x20,0x20,0x20,0x00,0x20,0x20,0x20,0x00,0x78,0x80,0x70,0x88,0x70,0x08,0xf0,0x00,
		0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x48,0x94,0xa4,0xa4,0x94,0x48,0x30,
		0x60,0x10,0x70,0x90,0x70,0x00,0x00,0x00,0x00,0x28,0x50,0xa0,0x50,0x28,0x00,0x00,
		0x00,0x00,0x00,0xf8,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x78,0x00,0x00,0x00,0x00,
		0x30,0x48,0xb4,0xb4,0xa4,0xb4,0x48,0x30,0x7c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x60,0x90,0x90,0x60,0x00,0x00,0x00,0x00,0x20,0x20,0xf8,0x20,0x20,0x00,0xf8,0x00,
		0x60,0x90,0x20,0x40,0xf0,0x00,0x00,0x00,0x60,0x90,0x20,0x90,0x60,0x00,0x00,0x00,
		0x10,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x88,0x88,0x88,0xc8,0xb0,0x80,
		0x78,0xd0,0xd0,0xd0,0x50,0x50,0x50,0x00,0x00,0x00,0x00,0x30,0x30,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x10,0x20,0x00,0x20,0x60,0x20,0x20,0x70,0x00,0x00,0x00,
		0x20,0x50,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0xa0,0x50,0x28,0x50,0xa0,0x00,0x00,
		0x40,0x48,0x50,0x28,0x58,0xa8,0x38,0x08,0x40,0x48,0x50,0x28,0x44,0x98,0x20,0x3c,
		0xc0,0x28,0xd0,0x28,0xd8,0xa8,0x38,0x08,0x20,0x00,0x20,0x40,0x80,0x88,0x70,0x00,
		0x40,0x20,0x70,0x88,0xf8,0x88,0x88,0x00,0x10,0x20,0x70,0x88,0xf8,0x88,0x88,0x00,
		0x70,0x00,0x70,0x88,0xf8,0x88,0x88,0x00,0x68,0xb0,0x70,0x88,0xf8,0x88,0x88,0x00,
		0x50,0x00,0x70,0x88,0xf8,0x88,0x88,0x00,0x20,0x50,0x70,0x88,0xf8,0x88,0x88,0x00,
		0x78,0xa0,0xa0,0xf0,0xa0,0xa0,0xb8,0x00,0x70,0x88,0x80,0x80,0x88,0x70,0x08,0x70,
		0x40,0x20,0xf8,0x80,0xf0,0x80,0xf8,0x00,0x10,0x20,0xf8,0x80,0xf0,0x80,0xf8,0x00,
		0x70,0x00,0xf8,0x80,0xf0,0x80,0xf8,0x00,0x50,0x00,0xf8,0x80,0xf0,0x80,0xf8,0x00,
		0x40,0x20,0x70,0x20,0x20,0x20,0x70,0x00,0x10,0x20,0x70,0x20,0x20,0x20,0x70,0x00,
		0x70,0x00,0x70,0x20,0x20,0x20,0x70,0x00,0x50,0x00,0x70,0x20,0x20,0x20,0x70,0x00,
		0x70,0x48,0x48,0xe8,0x48,0x48,0x70,0x00,0x68,0xb0,0x88,0xc8,0xa8,0x98,0x88,0x00,
		0x40,0x20,0x70,0x88,0x88,0x88,0x70,0x00,0x10,0x20,0x70,0x88,0x88,0x88,0x70,0x00,
		0x70,0x00,0x70,0x88,0x88,0x88,0x70,0x00,0x68,0xb0,0x70,0x88,0x88,0x88,0x70,0x00,
		0x50,0x00,0x70,0x88,0x88,0x88,0x70,0x00,0x00,0x88,0x50,0x20,0x50,0x88,0x00,0x00,
		0x00,0x74,0x88,0x90,0xa8,0x48,0xb0,0x00,0x40,0x20,0x88,0x88,0x88,0x88,0x70,0x00,
		0x10,0x20,0x88,0x88,0x88,0x88,0x70,0x00,0x70,0x00,0x88,0x88,0x88,0x88,0x70,0x00,
		0x50,0x00,0x88,0x88,0x88,0x88,0x70,0x00,0x10,0xa8,0x88,0x50,0x20,0x20,0x20,0x00,
		0x00,0x80,0xf0,0x88,0x88,0xf0,0x80,0x80,0x60,0x90,0x90,0xb0,0x88,0x88,0xb0,0x00,
		0x40,0x20,0x70,0x08,0x78,0x88,0x78,0x00,0x10,0x20,0x70,0x08,0x78,0x88,0x78,0x00,
		0x70,0x00,0x70,0x08,0x78,0x88,0x78,0x00,0x68,0xb0,0x70,0x08,0x78,0x88,0x78,0x00,
		0x50,0x00,0x70,0x08,0x78,0x88,0x78,0x00,0x20,0x50,0x70,0x08,0x78,0x88,0x78,0x00,
		0x00,0x00,0xf0,0x28,0x78,0xa0,0x78,0x00,0x00,0x00,0x70,0x88,0x80,0x78,0x08,0x70,
		0x40,0x20,0x70,0x88,0xf8,0x80,0x70,0x00,0x10,0x20,0x70,0x88,0xf8,0x80,0x70,0x00,
		0x70,0x00,0x70,0x88,0xf8,0x80,0x70,0x00,0x50,0x00,0x70,0x88,0xf8,0x80,0x70,0x00,
		0x40,0x20,0x00,0x60,0x20,0x20,0x70,0x00,0x10,0x20,0x00,0x60,0x20,0x20,0x70,0x00,
		0x20,0x50,0x00,0x60,0x20,0x20,0x70,0x00,0x50,0x00,0x00,0x60,0x20,0x20,0x70,0x00,
		0x50,0x60,0x10,0x78,0x88,0x88,0x70,0x00,0x68,0xb0,0x00,0xf0,0x88,0x88,0x88,0x00,
		0x40,0x20,0x00,0x70,0x88,0x88,0x70,0x00,0x10,0x20,0x00,0x70,0x88,0x88,0x70,0x00,
		0x20,0x50,0x00,0x70,0x88,0x88,0x70,0x00,0x68,0xb0,0x00,0x70,0x88,0x88,0x70,0x00,
		0x00,0x50,0x00,0x70,0x88,0x88,0x70,0x00,0x00,0x20,0x00,0xf8,0x00,0x20,0x00,0x00,
		0x00,0x00,0x68,0x90,0xa8,0x48,0xb0,0x00,0x40,0x20,0x88,0x88,0x88,0x98,0x68,0x00,
		0x10,0x20,0x88,0x88,0x88,0x98,0x68,0x00,0x70,0x00,0x88,0x88,0x88,0x98,0x68,0x00,
		0x50,0x00,0x88,0x88,0x88,0x98,0x68,0x00,0x10,0x20,0x88,0x88,0x88,0x78,0x08,0x70,
		0x80,0xf0,0x88,0x88,0xf0,0x80,0x80,0x80,0x50,0x00,0x88,0x88,0x88,0x78,0x08,0x70
	};

	static UINT8 fontdata12x8[] =
	{
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // dummy 0x00
		0x0f,0x00,0x19,0x80,0x36,0xc0,0x36,0xc0,0x30,0xc0,0x36,0xc0,0x16,0x80,0x0f,0x00, // BTN_A 0x01
		0x0f,0x00,0x11,0x80,0x36,0xc0,0x31,0xc0,0x36,0xc0,0x36,0xc0,0x10,0x80,0x0f,0x00, // BTN_B
		0x0f,0x00,0x19,0x80,0x36,0xc0,0x37,0xc0,0x37,0xc0,0x36,0xc0,0x19,0x80,0x0f,0x00, // BTN_C
		0x0f,0x00,0x11,0x80,0x36,0xc0,0x36,0xc0,0x36,0xc0,0x36,0xc0,0x11,0x80,0x0f,0x00, // BTN_D
		0x0f,0x00,0x11,0x80,0x36,0xc0,0x36,0xc0,0x31,0xc0,0x37,0xc0,0x17,0x80,0x0f,0x00, // BTN_P 0x05
		0x0f,0x00,0x16,0x80,0x35,0xc0,0x33,0xc0,0x33,0xc0,0x35,0xc0,0x16,0x80,0x0f,0x00, // BTN_K
		0x0f,0x00,0x19,0x80,0x36,0xc0,0x3b,0xc0,0x3d,0xc0,0x36,0xc0,0x19,0x80,0x0f,0x00, // BTN_S
		0x00,0x00,0x06,0x00,0x06,0x00,0x1f,0x80,0x1f,0x80,0x06,0x00,0x06,0x00,0x00,0x00, // BTN_+ 0x06
		0x0f,0x00,0x1d,0x80,0x31,0xc0,0x3d,0xc0,0x3d,0xc0,0x3d,0xc0,0x1d,0x80,0x0f,0x00, // BTN_1 0x07
		0x0f,0x00,0x19,0x80,0x36,0xc0,0x3e,0xc0,0x3d,0xc0,0x3b,0xc0,0x10,0x80,0x0f,0x00, // BTN_2
		0x0f,0x00,0x19,0x80,0x36,0xc0,0x3d,0xc0,0x3e,0xc0,0x36,0xc0,0x19,0x80,0x0f,0x00, // BTN_3
		0x0f,0x00,0x1d,0x80,0x39,0xc0,0x35,0xc0,0x2d,0xc0,0x20,0xc0,0x1d,0x80,0x0f,0x00, // BTN_4
		0x0f,0x00,0x10,0x80,0x37,0xc0,0x31,0xc0,0x3e,0xc0,0x36,0xc0,0x19,0x80,0x0f,0x00, // BTN_5
		0x0f,0x00,0x18,0x80,0x37,0xc0,0x31,0xc0,0x36,0xc0,0x36,0xc0,0x19,0x80,0x0f,0x00, // BTN_6
		0x0f,0x00,0x10,0x80,0x3e,0xc0,0x3d,0xc0,0x3b,0xc0,0x3b,0xc0,0x1b,0x80,0x0f,0x00, // BTN_7
		0x0f,0x00,0x19,0x80,0x36,0xc0,0x39,0xc0,0x36,0xc0,0x36,0xc0,0x19,0x80,0x0f,0x00, // BTN_8
		0x0f,0x00,0x19,0x80,0x36,0xc0,0x36,0xc0,0x38,0xc0,0x3e,0xc0,0x11,0x80,0x0f,0x00, // BTN_9
		0x0f,0x00,0x16,0x80,0x35,0x40,0x35,0x40,0x35,0x40,0x35,0x40,0x16,0x80,0x0f,0x00, // BTN_10
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x36,0xc0,0x36,0xc0,0x00,0x00, // DIR_... 0x13
		0x00,0x80,0x01,0xc0,0x03,0x80,0x07,0x00,0x2e,0x00,0x3c,0x00,0x38,0x00,0x3c,0x00, // DIR_1
		0x07,0x00,0x07,0x00,0x07,0x00,0x07,0x00,0x07,0x00,0x0f,0x80,0x07,0x00,0x02,0x00, // DIR_2
		0x10,0x00,0x38,0x00,0x1c,0x00,0x0e,0x00,0x07,0x40,0x03,0xc0,0x01,0xc0,0x03,0xc0, // DIR_3
		0x00,0x00,0x10,0x00,0x3f,0xc0,0x7f,0xc0,0x3f,0xc0,0x10,0x00,0x00,0x00,0x00,0x00, // DIR_4
		0x30,0xc0,0x38,0xc0,0x3c,0xc0,0x36,0xc0,0x33,0xc0,0x31,0xc0,0x30,0xc0,0x00,0x00, // DIR_N 0x17
		0x00,0x00,0x00,0x80,0x3f,0xc0,0x3f,0xe0,0x3f,0xc0,0x00,0x80,0x00,0x00,0x00,0x00, // DIR_6
		0x3c,0x00,0x38,0x00,0x3c,0x00,0x2e,0x00,0x07,0x00,0x03,0x80,0x01,0xc0,0x00,0x80, // DIR_7
		0x02,0x00,0x07,0x00,0x0f,0x80,0x07,0x00,0x07,0x00,0x07,0x00,0x07,0x00,0x07,0x00, // DIR_8
		0x03,0xc0,0x01,0xc0,0x03,0xc0,0x07,0x40,0x0e,0x00,0x1c,0x00,0x38,0x00,0x10,0x00, // DIR_9
		0x0F,0x00,0x1F,0x80,0x2C,0xC0,0x2D,0x40,0x2C,0xC0,0x25,0xC0,0x1F,0x80,0x0F,0x00, // LP                   _E
		0x0F,0x80,0x1F,0xC0,0x2A,0x60,0x22,0xA0,0x2A,0x60,0x2A,0xE0,0x1F,0xC0,0x0F,0x80, // MP                   _F
		0x0F,0x00,0x1F,0x80,0x24,0xC0,0x2D,0x40,0x34,0xC0,0x25,0xC0,0x1F,0x80,0x0F,0x00, // SP                   _G
		0x0F,0x00,0x1F,0x80,0x2D,0x40,0x2C,0xC0,0x2D,0x40,0x25,0x40,0x1F,0x80,0x0F,0x00, // LK                   _H
		0x0F,0x80,0x1F,0xC0,0x2A,0xA0,0x22,0x60,0x2A,0xA0,0x2A,0xA0,0x1F,0xC0,0x0F,0x80, // MK                   _I
		0x0F,0x00,0x1F,0x80,0x25,0x40,0x2C,0xC0,0x35,0x40,0x25,0x40,0x1F,0x80,0x0F,0x00, // SK                   _J
		0x0F,0x80,0x1F,0xC0,0x22,0xA0,0x32,0x60,0x3A,0xA0,0x22,0xA0,0x1F,0xC0,0x0F,0x80, // 3K                   _T
		0x0F,0x80,0x1F,0xC0,0x22,0x60,0x32,0xA0,0x3A,0x60,0x22,0xE0,0x1F,0xC0,0x0F,0x80, // 3P                   _U
		0x0F,0x80,0x1F,0xC0,0x26,0xA0,0x3A,0x60,0x36,0xA0,0x22,0xA0,0x1F,0xC0,0x0F,0x80, // 2K			   _V
		0x0F,0x80,0x1F,0xC0,0x26,0x60,0x3A,0xA0,0x36,0x60,0x22,0xE0,0x1F,0xC0,0x0F,0x80, // 2P			   _W
		0x00,0x00,0x00,0x00,0x74,0xC0,0x2A,0xA0,0x2E,0xC0,0x2A,0x80,0x00,0x00,0x00,0x00, // Tap                  _X
		0x0F,0x00,0x16,0x80,0x36,0xC0,0x32,0xC0,0x34,0xC0,0x36,0xC0,0x16,0x80,0x0F,0x00, // BTN_N                _Y
		0x0F,0x00,0x10,0x80,0x3E,0xC0,0x3D,0xC0,0x3B,0xC0,0x37,0xC0,0x10,0x80,0x0F,0x00, // BTN_Z                _Z
		0x0F,0x00,0x1F,0x80,0x3F,0xC0,0x3F,0xC0,0x3F,0xC0,0x3F,0xC0,0x1F,0x80,0x0F,0x00, // Joystick Ball	   _5
		0x00,0x00,0x00,0x00,0x06,0x00,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // Small Dot            _'
		0x0F,0x00,0x10,0x80,0x26,0x40,0x29,0x40,0x29,0x40,0x26,0x40,0x10,0x80,0x0F,0x00, // Double Ball          _@
		0x0F,0x00,0x10,0x80,0x20,0x40,0x20,0x40,0x20,0x40,0x20,0x40,0x10,0x80,0x0F,0x00, // Single Ball          _)
		0x0F,0x00,0x1F,0x80,0x3F,0xC0,0x3F,0xC0,0x3F,0xC0,0x3F,0xC0,0x1F,0x80,0x0F,0x00, // Solid Ball           _(
		0x00,0x00,0x00,0x80,0x00,0x40,0x3F,0xE0,0x00,0x40,0x00,0x80,0x00,0x00,0x00,0x00, // Arrow                _!
		0x02,0x00,0x05,0x00,0x3D,0xE0,0x10,0x40,0x08,0x80,0x0A,0x80,0x0D,0x80,0x00,0x00, // Star                 _*
		0x02,0x00,0x07,0x00,0x3F,0xE0,0x1F,0xC0,0x0F,0x80,0x0F,0x80,0x0D,0x80,0x00,0x00, // Solid star           _&
		0x00,0x00,0x02,0x00,0x05,0x00,0x08,0x80,0x10,0x40,0x20,0x20,0x3F,0xE0,0x00,0x00, // Triangle             _^
		0x00,0x00,0x02,0x00,0x07,0x00,0x0F,0x80,0x1F,0xC0,0x3F,0xE0,0x3F,0xE0,0x00,0x00, // Solid Triangle       _%
		0x1F,0xC0,0x10,0x40,0x10,0x40,0x10,0x40,0x10,0x40,0x10,0x40,0x1F,0xC0,0x00,0x00, // Square               _$
		0x1F,0xC0,0x1F,0xC0,0x1F,0xC0,0x1F,0xC0,0x1F,0xC0,0x1F,0xC0,0x1F,0xC0,0x00,0x00, // Solid Square         _#
		0x00,0x00,0x3F,0xE0,0x20,0x20,0x10,0x40,0x08,0x80,0x05,0x00,0x02,0x00,0x00,0x00, // Down Triangle        _<
		0x00,0x00,0x3F,0xE0,0x3F,0xE0,0x1F,0xC0,0x0F,0x80,0x07,0x00,0x02,0x00,0x00,0x00, // Solid Down Triangle  _>
		0x02,0x00,0x05,0x00,0x08,0x80,0x10,0x40,0x08,0x80,0x05,0x00,0x02,0x00,0x00,0x00, // Diamond              _{
		0x02,0x00,0x07,0x00,0x0F,0x80,0x1F,0xC0,0x0F,0x80,0x07,0x00,0x02,0x00,0x00,0x00, // Solid Diamond        _}
		0x08,0x00,0x1C,0x00,0x3E,0xE0,0x1C,0xE0,0x1F,0xE0,0x0F,0xC0,0x07,0x80,0x00,0x00, // Half Circle Back     _k
		0x0F,0x00,0x1F,0x80,0x3F,0xC0,0x39,0xC0,0x3B,0xE0,0x01,0xC0,0x00,0x80,0x00,0x00, // Half Circle Front Up _l
		0x00,0x80,0x01,0xC0,0x3B,0xE0,0x39,0xC0,0x3F,0xC0,0x1F,0x80,0x0F,0x00,0x00,0x00, // Half Circle Front    _m
		0x0F,0x00,0x1F,0x80,0x3F,0xC0,0x39,0xC0,0x7D,0xC0,0x38,0x00,0x10,0x00,0x00,0x00, // Half Circle Back Up  _n
		0x01,0xC0,0x01,0xC0,0x05,0xC0,0x0F,0xC0,0x1F,0x80,0x0F,0x00,0x04,0x00,0x00,0x00, // 1/4 Cir For 2 Down   _o
		0x04,0x00,0x0E,0x00,0x1F,0x00,0x0E,0x00,0x0F,0xC0,0x07,0xC0,0x03,0xC0,0x00,0x00, // 1/4 Cir Down 2 Back  _p
		0x01,0x00,0x07,0x80,0x0F,0xC0,0x1F,0x80,0x1D,0x00,0x1C,0x00,0x1C,0x00,0x00,0x00, // 1/4 Cir Back 2 Up    _q
		0x1E,0x00,0x1F,0x00,0x1F,0x80,0x03,0x80,0x07,0xC0,0x03,0x80,0x01,0x00,0x00,0x00, // 1/4 Cir Up 2 For     _r
		0x1C,0x00,0x1C,0x00,0x1D,0x00,0x1F,0x80,0x0F,0xC0,0x07,0x80,0x01,0x00,0x00,0x00, // 1/4 Cir Back 2 Down  _s
		0x01,0x00,0x03,0x80,0x07,0xC0,0x03,0x80,0x1F,0x80,0x1F,0x00,0x1E,0x00,0x00,0x00, // 1/4 Cir Down 2 For   _t
		0x04,0x00,0x0F,0x00,0x1F,0x80,0x0F,0xC0,0x05,0xC0,0x01,0xC0,0x01,0xC0,0x00,0x00, // 1/4 Cir For 2 Up     _u
		0x03,0xC0,0x07,0xC0,0x0F,0xC0,0x0E,0x00,0x1F,0x00,0x0E,0x00,0x04,0x00,0x00,0x00, // 1/4 Cir Up 2 Back    _v
		0x0F,0x00,0x1F,0x80,0x3F,0xC0,0x39,0xC0,0x3B,0xE0,0x3D,0xC0,0x1E,0x80,0x0C,0x00, // Full Clock Forward   _w
		0x01,0x80,0x0B,0xC0,0x1D,0xE0,0x3E,0xE0,0x1C,0xE0,0x1F,0xE0,0x0F,0xC0,0x07,0x80, // Full Clock Back      _x
		0x0C,0x00,0x1E,0x80,0x3D,0xC0,0x3B,0xE0,0x39,0xC0,0x3F,0xC0,0x1F,0x80,0x0F,0x00, // Full Count Forward   _y
		0x0F,0x00,0x1F,0x80,0x3F,0xC0,0x39,0xC0,0x7D,0xC0,0x3B,0xC0,0x17,0x80,0x03,0x00, // Full Count Back      _z
		0x00,0x00,0x08,0x80,0x7D,0xC0,0x7E,0xE0,0x7D,0xC0,0x08,0x80,0x00,0x00,0x00,0x00, // 2x Forward		   _L
		0x00,0x00,0x11,0x00,0x3B,0xE0,0x77,0xE0,0x3B,0xE0,0x11,0x00,0x00,0x00,0x00,0x00, // 2x Back		   _M	
		0x02,0x00,0x07,0x00,0x0F,0x80,0x07,0x00,0x07,0x00,0x07,0x00,0x00,0x00,0x0F,0x80, // Jump                 _|
		0x02,0x00,0x02,0x00,0x02,0x00,0x0A,0x80,0x07,0x00,0x0A,0x80,0x1D,0xC0,0x0F,0x80, // Hold                 _O
		0x00,0x00,0x00,0x00,0x32,0xC0,0x2A,0xA0,0x2A,0xC0,0x32,0xA0,0x00,0x00,0x00,0x00, // DIR                  _?
		0x00,0x00,0x3F,0x00,0x3E,0x00,0x3C,0x80,0x0F,0xC0,0x1F,0xE0,0x3F,0xC0,0x00,0x80, // Dragon Punch Forward _Q
		0x00,0x00,0x07,0xE0,0x03,0xE0,0x09,0xE0,0x1F,0x80,0x3F,0xC0,0x1F,0xE0,0x08,0x00, // Dragon Punch Back    _R
		0x0F,0x80,0x00,0x00,0x02,0x00,0x07,0x00,0x0A,0x80,0x02,0x00,0x02,0x00,0x00,0x00, // Air                  _=
		0x02,0x00,0x02,0x00,0x0A,0x80,0x07,0x00,0x02,0x00,0x00,0x00,0x0F,0x80,0x00,0x00, // Squatting            _-
		0x00,0x00,0x04,0x40,0x02,0x40,0x3F,0x40,0x02,0x40,0x04,0x40,0x00,0x00,0x00,0x00, // Close                _]
		0x00,0x00,0x08,0x40,0x10,0x40,0x3F,0x40,0x10,0x40,0x08,0x40,0x00,0x00,0x00,0x00, // Away                 _[
		0x02,0x00,0x42,0x10,0x22,0x20,0x0A,0x80,0x67,0x30,0x0A,0x80,0x1D,0xC0,0x0F,0x80, // Charge               _~	
	};

	static unsigned char fontdata6x12[] =
	{
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x04,0x14,0x34,0x7c,0x30,0x10,0x00,0x00,0x00,
		0x00,0x00,0x7c,0x80,0x98,0x90,0x80,0xbc,0x80,0x7c,0x00,0x00,
		0x00,0x00,0xf8,0x04,0x64,0x44,0x04,0xf4,0x04,0xf8,0x00,0x00,
		0x00,0x00,0x7c,0x80,0x98,0x88,0x80,0xbc,0x80,0x7c,0x00,0x00,
		0x00,0x00,0xf8,0x04,0x64,0x24,0x04,0xf4,0x04,0xf8,0x00,0x00,
		0x00,0x00,0x7c,0x80,0x88,0x98,0x80,0xbc,0x80,0x7c,0x00,0x00,
		0x00,0x00,0xf8,0x04,0x24,0x64,0x04,0xf4,0x04,0xf8,0x00,0x00,
		0x00,0x00,0x7c,0x80,0x90,0x98,0x80,0xbc,0x80,0x7c,0x00,0x00,
		0x00,0x00,0xf8,0x04,0x44,0x64,0x04,0xf4,0x04,0xf8,0x00,0x00,
		0x00,0x00,0x10,0x28,0x44,0x54,0x54,0x44,0x28,0x10,0x00,0x00,
		0x00,0x00,0x10,0x28,0x44,0x44,0x44,0x44,0x28,0x10,0x00,0x00,
		0x00,0x00,0x00,0x7e,0x42,0x46,0x6a,0x52,0x7e,0x00,0x00,0x00,
		0x00,0x00,0x00,0x7e,0x42,0x42,0x42,0x42,0x7e,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x30,0x68,0x78,0x78,0x30,0x00,0x00,0x00,0x00,
		0x00,0x00,0x80,0xc0,0xe0,0xf0,0xf0,0xe0,0xc0,0x80,0x00,0x00,
		0x00,0x00,0x08,0x18,0x38,0x78,0x78,0x38,0x18,0x08,0x00,0x00,
		0x00,0x00,0x20,0x70,0xf8,0x20,0x20,0xf8,0x70,0x20,0x00,0x00,
		0x50,0x50,0x50,0x50,0x50,0x50,0x50,0x00,0x00,0x50,0x00,0x00,
		0x00,0x00,0x00,0x00,0x60,0xd0,0xf0,0x60,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x60,0xd0,0xf0,0xf0,0x60,0x00,0x00,0x00,0x00,
		0x00,0x00,0x70,0xd8,0xe8,0xe8,0xf8,0xf8,0x70,0x00,0x00,0x00,
		0x00,0x00,0x18,0x78,0x68,0x48,0x48,0x58,0xd8,0xc0,0x00,0x00,
		0x00,0x00,0x30,0x78,0xfc,0x78,0x78,0x78,0x78,0x00,0x00,0x00,
		0x00,0x00,0x78,0x78,0x78,0x78,0xfc,0x78,0x30,0x00,0x00,0x00,
		0x00,0x00,0x00,0x10,0xf8,0xfc,0xfc,0xf8,0x10,0x00,0x00,0x00,
		0x00,0x00,0x00,0x20,0x7c,0xfc,0xfc,0x7c,0x20,0x00,0x00,0x00,
		0x00,0x00,0x00,0xb0,0x54,0xb8,0xb8,0x54,0xb0,0x00,0x00,0x00,
		0x00,0x00,0x00,0x28,0x6c,0xfc,0x6c,0x28,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x30,0x30,0x78,0x78,0xfc,0xfc,0x00,0x00,0x00,
		0x0e,0x00,0x3f,0x80,0x71,0xc0,0x6e,0xc0,0xee,0xe0,0xee,0xe0,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x00,0x00,0x20,0x00,0x00,
		0xa0,0xa0,0xa0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x50,0x50,0xf8,0x50,0x50,0x50,0xf8,0x50,0x50,0x00,0x00,
		0x00,0x20,0x78,0xa0,0xa0,0x70,0x28,0x28,0xf0,0x20,0x00,0x00,
		0x40,0xa0,0xa4,0x48,0x10,0x20,0x48,0x94,0x14,0x08,0x00,0x00,
		0x00,0x60,0x90,0xa0,0x40,0xa8,0xa8,0x90,0x90,0x68,0x00,0x00,
		0x10,0x20,0x40,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x10,0x20,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x20,0x10,0x00,
		0x40,0x20,0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x20,0x40,0x00,
		0x00,0x00,0x20,0xa8,0x70,0xf8,0x70,0xa8,0x20,0x00,0x00,0x00,
		0x00,0x00,0x20,0x20,0x20,0xf8,0x20,0x20,0x20,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x30,0x60,0x00,
		0x00,0x00,0x00,0x00,0x00,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x60,0x00,0x00,
		0x08,0x08,0x10,0x10,0x20,0x20,0x40,0x40,0x80,0x80,0x00,0x00,
		0x00,0x70,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x70,0x00,0x00,
		0x00,0x20,0x60,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x00,0x00,
		0x00,0x70,0x88,0x88,0x08,0x10,0x20,0x40,0x80,0xf8,0x00,0x00,
		0x00,0x70,0x88,0x88,0x08,0x70,0x08,0x88,0x88,0x70,0x00,0x00,
		0x00,0x30,0x30,0x50,0x50,0x90,0x90,0xf8,0x10,0x10,0x00,0x00,
		0x00,0xf8,0x80,0x80,0xf0,0x08,0x08,0x88,0x88,0x70,0x00,0x00,
		0x00,0x70,0x80,0x80,0xf0,0x88,0x88,0x88,0x88,0x70,0x00,0x00,
		0x00,0xf8,0x08,0x08,0x08,0x10,0x20,0x20,0x20,0x20,0x00,0x00,
		0x00,0x70,0x88,0x88,0x88,0x70,0x88,0x88,0x88,0x70,0x00,0x00,
		0x00,0x70,0x88,0x88,0x88,0x88,0x78,0x08,0x08,0x70,0x00,0x00,
		0x00,0x00,0x00,0x60,0x60,0x00,0x00,0x00,0x60,0x60,0x00,0x00,
		0x00,0x00,0x00,0x60,0x60,0x00,0x00,0x00,0x60,0x20,0x40,0x00,
		0x00,0x08,0x10,0x20,0x40,0x80,0x40,0x20,0x10,0x08,0x00,0x00,
		0x00,0x00,0x00,0xf0,0x00,0x00,0xf0,0x00,0x00,0x00,0x00,0x00,
		0x00,0x80,0x40,0x20,0x10,0x08,0x10,0x20,0x40,0x80,0x00,0x00,
		0x00,0x70,0x88,0x88,0x08,0x10,0x20,0x20,0x00,0x20,0x00,0x00,
		0x00,0x70,0x88,0xe8,0xa8,0xa8,0xe8,0x90,0x88,0x70,0x00,0x00,
		0x00,0x70,0x88,0x88,0x88,0x88,0xf8,0x88,0x88,0x88,0x00,0x00,
		0x00,0xf0,0x88,0x88,0x88,0xf0,0x88,0x88,0x88,0xf0,0x00,0x00,
		0x00,0x70,0x88,0x88,0x80,0x80,0x80,0x88,0x88,0x70,0x00,0x00,
		0x00,0xf0,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0xf0,0x00,0x00,
		0x00,0xf8,0x80,0x80,0x80,0xf0,0x80,0x80,0x80,0xf8,0x00,0x00,
		0x00,0xf8,0x80,0x80,0x80,0xf0,0x80,0x80,0x80,0x80,0x00,0x00,
		0x00,0x70,0x88,0x88,0x80,0x98,0x88,0x88,0x88,0x70,0x00,0x00,
		0x00,0x88,0x88,0x88,0x88,0xf8,0x88,0x88,0x88,0x88,0x00,0x00,
		0x00,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x00,0x00,
		0x00,0x08,0x08,0x08,0x08,0x08,0x08,0x88,0x88,0x70,0x00,0x00,
		0x00,0x88,0x88,0x90,0xa0,0xc0,0xa0,0x90,0x88,0x88,0x00,0x00,
		0x00,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0xf8,0x00,0x00,
		0x00,0x88,0x88,0xd8,0xd8,0xa8,0xa8,0x88,0x88,0x88,0x00,0x00,
		0x00,0x88,0xc8,0xc8,0xa8,0xa8,0x98,0x98,0x88,0x88,0x00,0x00,
		0x00,0x70,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x70,0x00,0x00,
		0x00,0xf0,0x88,0x88,0x88,0xf0,0x80,0x80,0x80,0x80,0x00,0x00,
		0x00,0x70,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x70,0x08,0x00,
		0x00,0xf0,0x88,0x88,0x88,0xf0,0x88,0x88,0x88,0x88,0x00,0x00,
		0x00,0x70,0x88,0x88,0x80,0x70,0x08,0x88,0x88,0x70,0x00,0x00,
		0x00,0xf8,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x00,0x00,
		0x00,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x70,0x00,0x00,
		0x00,0x88,0x88,0x88,0x88,0x88,0x88,0x50,0x50,0x20,0x00,0x00,
		0x00,0x88,0x88,0x88,0x88,0xa8,0xa8,0xd8,0xd8,0x88,0x00,0x00,
		0x00,0x88,0x50,0x50,0x20,0x20,0x20,0x50,0x50,0x88,0x00,0x00,
		0x00,0x88,0x88,0x88,0x50,0x50,0x20,0x20,0x20,0x20,0x00,0x00,
		0x00,0xf8,0x08,0x10,0x10,0x20,0x40,0x40,0x80,0xf8,0x00,0x00,
		0x38,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x38,0x00,
		0x00,0x88,0x88,0x50,0x50,0xf8,0x20,0xf8,0x20,0x20,0x00,0x00,
		0xe0,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0xe0,0x00,
		0x20,0x50,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfc,0x00,
		0x40,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x70,0x88,0x78,0x88,0x88,0x78,0x00,0x00,
		0x00,0x80,0x80,0x80,0xf0,0x88,0x88,0x88,0x88,0xf0,0x00,0x00,
		0x00,0x00,0x00,0x00,0x70,0x88,0x80,0x80,0x88,0x70,0x00,0x00,
		0x00,0x08,0x08,0x08,0x78,0x88,0x88,0x88,0x88,0x78,0x00,0x00,
		0x00,0x00,0x00,0x00,0x70,0x88,0xf8,0x80,0x88,0x70,0x00,0x00,
		0x00,0x30,0x40,0x40,0xe0,0x40,0x40,0x40,0x40,0x40,0x00,0x00,
		0x00,0x00,0x00,0x00,0x78,0x88,0x88,0x78,0x08,0x88,0x70,0x00,
		0x00,0x80,0x80,0x80,0xf0,0x88,0x88,0x88,0x88,0x88,0x00,0x00,
		0x00,0x20,0x20,0x00,0x20,0x20,0x20,0x20,0x20,0x20,0x00,0x00,
		0x00,0x20,0x20,0x00,0x20,0x20,0x20,0x20,0x20,0x20,0xc0,0x00,
		0x00,0x80,0x80,0x80,0x80,0x90,0xa0,0xe0,0x90,0x88,0x00,0x00,
		0x00,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x00,0x00,
		0x00,0x00,0x00,0x00,0xf0,0xa8,0xa8,0xa8,0xa8,0xa8,0x00,0x00,
		0x00,0x00,0x00,0x00,0xb0,0xc8,0x88,0x88,0x88,0x88,0x00,0x00,
		0x00,0x00,0x00,0x00,0x70,0x88,0x88,0x88,0x88,0x70,0x00,0x00,
		0x00,0x00,0x00,0x00,0xf0,0x88,0x88,0x88,0xf0,0x80,0x80,0x00,
		0x00,0x00,0x00,0x00,0x78,0x88,0x88,0x88,0x78,0x08,0x08,0x00,
		0x00,0x00,0x00,0x00,0xb0,0xc8,0x80,0x80,0x80,0x80,0x00,0x00,
		0x00,0x00,0x00,0x00,0x70,0x88,0x60,0x10,0x88,0x70,0x00,0x00,
		0x00,0x40,0x40,0x40,0xe0,0x40,0x40,0x40,0x40,0x30,0x00,0x00,
		0x00,0x00,0x00,0x00,0x88,0x88,0x88,0x88,0x98,0x68,0x00,0x00,
		0x00,0x00,0x00,0x00,0x88,0x88,0x88,0x88,0x50,0x20,0x00,0x00,
		0x00,0x00,0x00,0x00,0xa8,0xa8,0xa8,0xa8,0xa8,0x50,0x00,0x00,
		0x00,0x00,0x00,0x00,0x88,0x50,0x20,0x20,0x50,0x88,0x00,0x00,
		0x00,0x00,0x00,0x00,0x88,0x88,0x88,0x88,0x78,0x08,0x70,0x00,
		0x00,0x00,0x00,0x00,0xf8,0x08,0x10,0x20,0x40,0xf8,0x00,0x00,
		0x10,0x20,0x20,0x20,0x20,0x40,0x20,0x20,0x20,0x20,0x10,0x00,
		0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x00,
		0x40,0x20,0x20,0x20,0x20,0x10,0x20,0x20,0x20,0x20,0x40,0x00,
		0x68,0x90,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x20,0x50,0x20,0x50,0xa8,0x50,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xa8,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x50,0x50,0x20,0x00,0x00,
		0x70,0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x00,0x00,0x00,0x00,
		0x00,0x00,0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x70,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x40,0x20,0x00,0x00,
		0x00,0x00,0x00,0x00,0x30,0x30,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0xf8,0x08,0x08,0xf8,0x08,0x08,0x10,0x20,0xc0,0x00,0x00,
		0x00,0x00,0x00,0x00,0xf8,0x08,0x28,0x30,0x20,0x40,0x00,0x00,
		0x00,0x00,0x00,0x00,0x10,0x20,0x60,0xa0,0x20,0x20,0x00,0x00,
		0x00,0x00,0x00,0x00,0x20,0xf8,0x88,0x08,0x10,0x20,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0xf8,0x20,0x20,0x20,0xf8,0x00,0x00,
		0x00,0x00,0x00,0x10,0x10,0xf8,0x10,0x30,0x50,0x90,0x00,0x00,
		0x00,0x00,0x00,0x40,0x40,0xf8,0x48,0x50,0x40,0x40,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x70,0x10,0x10,0x10,0xf8,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0xf0,0x10,0xf0,0x10,0xf0,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0xa8,0xa8,0x08,0x10,0x60,0x00,0x00,
		0x00,0x00,0x00,0x00,0x80,0x78,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0xf8,0x08,0x28,0x28,0x30,0x20,0x20,0x40,0x80,0x00,0x00,
		0x00,0x08,0x10,0x10,0x20,0x60,0xa0,0x20,0x20,0x20,0x00,0x00,
		0x00,0x20,0x20,0xf8,0x88,0x88,0x10,0x10,0x20,0x40,0x00,0x00,
		0x00,0x00,0xf8,0x20,0x20,0x20,0x20,0x20,0x20,0xf8,0x00,0x00,
		0x00,0x10,0xf8,0x10,0x30,0x30,0x50,0x50,0x90,0x30,0x00,0x00,
		0x00,0x20,0x20,0xf8,0x28,0x28,0x28,0x48,0x48,0x98,0x00,0x00,
		0x00,0x20,0x20,0xf8,0x20,0x20,0xf8,0x20,0x20,0x20,0x00,0x00,
		0x00,0x40,0x78,0x48,0x88,0x88,0x10,0x10,0x20,0x40,0x00,0x00,
		0x00,0x40,0x78,0x50,0x90,0x90,0x10,0x20,0x20,0x40,0x00,0x00,
		0x00,0x00,0xf8,0x08,0x08,0x08,0x08,0x08,0x08,0xf8,0x00,0x00,
		0x00,0x50,0xf8,0x50,0x50,0x50,0x10,0x20,0x20,0x40,0x00,0x00,
		0x00,0x80,0x48,0x08,0x88,0x48,0x08,0x10,0x20,0xc0,0x00,0x00,
		0x00,0xf0,0x10,0x10,0x10,0x20,0x20,0x50,0x48,0x88,0x00,0x00,
		0x00,0x40,0x40,0xf8,0x48,0x48,0x50,0x40,0x40,0x38,0x00,0x00,
		0x00,0x88,0x88,0x48,0x48,0x08,0x10,0x10,0x20,0x40,0x00,0x00,
		0x00,0x40,0x78,0x48,0x48,0xa8,0x98,0x10,0x20,0x40,0x00,0x00,
		0x00,0x10,0x60,0x20,0xf8,0x20,0x20,0x40,0x40,0x80,0x00,0x00,
		0x00,0xa0,0xa8,0xa8,0x08,0x08,0x10,0x10,0x20,0x40,0x00,0x00,
		0x00,0x70,0x00,0x00,0xf8,0x20,0x20,0x20,0x40,0x80,0x00,0x00,
		0x00,0x40,0x40,0x40,0x60,0x50,0x48,0x40,0x40,0x40,0x00,0x00,
		0x00,0x20,0x20,0xf8,0x20,0x20,0x20,0x20,0x40,0x80,0x00,0x00,
		0x00,0x00,0x70,0x00,0x00,0x00,0x00,0x00,0x00,0xf8,0x00,0x00,
		0x00,0x00,0xf8,0x08,0x48,0x30,0x10,0x28,0x48,0x80,0x00,0x00,
		0x00,0x20,0xf8,0x08,0x08,0x10,0x20,0x68,0xa8,0x20,0x00,0x00,
		0x00,0x08,0x08,0x08,0x08,0x10,0x10,0x20,0x20,0x40,0x00,0x00,
		0x00,0x50,0x50,0x50,0x48,0x48,0x48,0x88,0x88,0x88,0x00,0x00,
		0x00,0x80,0x80,0xf8,0x80,0x80,0x80,0x80,0x80,0x78,0x00,0x00,
		0x00,0xf8,0x08,0x08,0x08,0x08,0x10,0x10,0x20,0x40,0x00,0x00,
		0x00,0x40,0x40,0xa0,0xa0,0x10,0x10,0x08,0x08,0x00,0x00,0x00,
		0x00,0x20,0xf8,0x20,0x20,0xa8,0xa8,0xa8,0xa8,0x20,0x00,0x00,
		0x00,0x00,0xf8,0x08,0x08,0x10,0x90,0x60,0x20,0x10,0x00,0x00,
		0x00,0xc0,0x30,0x00,0xc0,0x30,0x00,0xc0,0x30,0x00,0x00,0x00,
		0x00,0x40,0x40,0x40,0x40,0x40,0x50,0x88,0x88,0xf8,0x00,0x00,
		0x00,0x10,0x10,0x90,0x50,0x20,0x30,0x28,0x40,0x80,0x00,0x00,
		0x00,0xf8,0x20,0x20,0x20,0xf8,0x20,0x20,0x20,0x18,0x00,0x00,
		0x00,0x40,0x40,0xf8,0x48,0x48,0x50,0x40,0x40,0x40,0x00,0x00,
		0x00,0x00,0x00,0x70,0x10,0x10,0x10,0x10,0x10,0xf8,0x00,0x00,
		0x00,0x00,0x00,0xf8,0x08,0x08,0xf8,0x08,0x08,0xf8,0x00,0x00,
		0x00,0x70,0x00,0xf8,0x08,0x08,0x08,0x10,0x10,0x20,0x00,0x00,
		0x00,0x90,0x90,0x90,0x90,0x10,0x10,0x20,0x20,0x40,0x00,0x00,
		0x00,0xa0,0xa0,0xa0,0xa0,0xa0,0xa8,0xa8,0xb0,0xa0,0x00,0x00,
		0x00,0x80,0x80,0x80,0x80,0x80,0x88,0x90,0xa0,0xc0,0x00,0x00,
		0x00,0x00,0xf8,0x88,0x88,0x88,0x88,0x88,0x88,0xf8,0x00,0x00,
		0x00,0xf8,0x88,0x88,0x88,0x08,0x10,0x10,0x20,0x40,0x00,0x00,
		0x00,0x00,0x80,0x48,0x08,0x08,0x10,0x10,0x20,0xc0,0x00,0x00,
		0x00,0xa0,0xa0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x40,0xa0,0xa0,0x40,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	};

	static const struct GfxLayout fontlayout6x8 =
	{
		6,8,
		256,
		1,
		{ 0 },
		{ 0, 1, 2, 3, 4, 5, 6, 7 },
		{ 0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 6*8, 7*8 },
		8*8
	};

	static const struct GfxLayout fontlayout12x8 =
	{
		12,8,
		COLOR_BUTTONS,
		1,
		{ 0 },
		{ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15 },
		{ 0*16, 1*16, 2*16, 3*16, 4*16, 5*16, 6*16, 7*16 },
		8*16
	};

	static struct GfxLayout fontlayout6x12 =
	{
		6,12,	/* 6*12 characters */
		256,    /* 256 characters */
		1,	/* 1 bit per pixel */
		{ 0 },
		{ 0, 1, 2, 3, 4, 5, 6, 7 },	/* straightforward layout */
		{ 0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 6*8, 7*8, 8*8, 9*8, 10*8, 11*8 },
		8*12	/* every char takes 8 consecutive bytes */
	};
	static struct GfxLayout fontlayout12x12 =
	{
		12,12,
		0,	// total number set later
		1,
		{ 0 },
		{ 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15 },
		{ 0*16,1*16,2*16,3*16,4*16,5*16,6*16,7*16,8*16,9*16,10*16,11*16 },
		16*12
	};

	struct GfxLayout layout;
	struct GfxLayout layout1;
	struct GfxLayout layout2;
	unsigned char *fontdata_sb;
	UINT32 tempoffset[MAX_GFX_SIZE];
	static int fontlangcode = -1;
	static int use_8linefont = 1;
	int temp, i;

	uifont_freefont();

	if (fontlangcode != options.langcode)
	{
		if (!use_8linefont && fontdata_db)
			free(fontdata_db);

		if ((fontdata_db = load_fontdata()) == NULL)
			use_8linefont = 1;
		else
			use_8linefont = 0;

		fontlangcode = options.langcode;
	}

	if (use_8linefont)
	{
		fontdata_sb = fontdata6x8;
		fontdata_db = fontdata12x8;

		layout = fontlayout6x8;
		layout1 = fontlayout6x8;
		layout2 = fontlayout12x8;
	}
	else
	{
		fontdata_sb = fontdata6x12;

		layout = fontlayout6x12;
		layout1 = fontlayout6x12;
		layout2 = fontlayout12x12;
		layout2.total = ui_lang_info[options.langcode].numchars;
	}


gamescale = options.artwork_res;
if (gamescale == 2)
{
	if (Machine->uiwidth >= 420)
	{
		if (Machine->ui_orientation & ORIENTATION_SWAP_XY)
		{
			/* pixel double vertically */
			memcpy(tempoffset, layout.yoffset, sizeof(tempoffset));
			for (i = 0; i < layout.height; i++)
				layout.yoffset[i*2+0] = layout.yoffset[i*2+1] = tempoffset[i];
			layout.height *= 2;
		}
		else
		{
			/* pixel double horizontally */
			memcpy(tempoffset, layout.xoffset, sizeof(tempoffset));
			for (i = 0; i < layout.width; i++)
				layout.xoffset[i*2+0] = layout.xoffset[i*2+1] = tempoffset[i];
			layout.width *= 2;
		}
	}

	if (Machine->uiheight >= 420)
	{
		if (Machine->ui_orientation & ORIENTATION_SWAP_XY)
		{
			/* pixel double horizontally */
			memcpy(tempoffset, layout.xoffset, sizeof(tempoffset));
			for (i = 0; i < layout.width; i++)
				layout.xoffset[i*2+0] = layout.xoffset[i*2+1] = tempoffset[i];
			layout.width *= 2;
		}
		else
		{
			/* pixel double vertically */
			memcpy(tempoffset, layout.yoffset, sizeof(tempoffset));
			for (i = 0; i < layout.height; i++)
				layout.yoffset[i*2+0] = layout.yoffset[i*2+1] = tempoffset[i];
			layout.height *= 2;
		}
	}
}

	/* decode a straight on version for games */
	Machine->uifont = decodegfx(fontdata_sb, &layout);
	if (!Machine->uifont)
		return 0;

	Machine->uifontwidth = layout.width;
	Machine->uifontheight = layout.height;

	Machine->uifont->colortable = colortable;
	Machine->uifont->total_colors = MAX_COLORTABLE;

	/* apply swappage */
	if (Machine->ui_orientation & ORIENTATION_SWAP_XY)
	{
		memcpy(tempoffset, layout1.xoffset, sizeof(tempoffset));
		memcpy(layout1.xoffset, layout1.yoffset, sizeof(layout1.xoffset));
		memcpy(layout1.yoffset, tempoffset, sizeof(layout1.yoffset));

		temp = layout1.width;
		layout1.width = layout1.height;
		layout1.height = temp;

		memcpy(tempoffset, layout2.xoffset, sizeof(tempoffset));
		memcpy(layout2.xoffset, layout2.yoffset, sizeof(layout2.xoffset));
		memcpy(layout2.yoffset, tempoffset, sizeof(layout2.yoffset));

		temp = layout2.width;
		layout2.width = layout2.height;
		layout2.height = temp;
	}

	/* apply xflip */
	if (Machine->ui_orientation & ORIENTATION_FLIP_X)
	{
		memcpy(tempoffset, layout1.xoffset, sizeof(tempoffset));
		for (i = 0; i < layout1.width; i++)
			layout1.xoffset[i] = tempoffset[layout1.width - 1 - i];

		memcpy(tempoffset, layout2.xoffset, sizeof(tempoffset));
		for (i = 0; i < layout2.width; i++)
			layout2.xoffset[i] = tempoffset[layout2.width - 1 - i];
	}

	/* apply yflip */
	if (Machine->ui_orientation & ORIENTATION_FLIP_Y)
	{
		memcpy(tempoffset, layout1.yoffset, sizeof(tempoffset));
		for (i = 0; i < layout1.height; i++)
			layout1.yoffset[i] = tempoffset[layout1.height - 1 - i];

		memcpy(tempoffset, layout2.yoffset, sizeof(tempoffset));
		for (i = 0; i < layout2.height; i++)
			layout2.yoffset[i] = tempoffset[layout2.height - 1 - i];
	}

gamescale = options.artwork_res;
if (gamescale == 2)
{
	/* pixel double horizontally */
	if (Machine->uiwidth >= 420)
	{
		memcpy(tempoffset, layout1.xoffset, sizeof(tempoffset));
		for (i = 0; i < layout1.width; i++)
			layout1.xoffset[i*2+0] = layout1.xoffset[i*2+1] = tempoffset[i];
		layout1.width *= 2;

		memcpy(tempoffset, layout2.xoffset, sizeof(tempoffset));
		for (i = 0; i < layout2.width; i++)
			layout2.xoffset[i*2+0] = layout2.xoffset[i*2+1] = tempoffset[i];
		layout2.width *= 2;
	}

	/* pixel double vertically */
	if (Machine->uiheight >= 420)
	{
		memcpy(tempoffset, layout1.yoffset, sizeof(tempoffset));
		for (i = 0; i < layout1.height; i++)
			layout1.yoffset[i*2+0] = layout1.yoffset[i*2+1] = tempoffset[i];
		layout1.height *= 2;

		memcpy(tempoffset, layout2.yoffset, sizeof(tempoffset));
		for (i = 0; i < layout2.height; i++)
			layout2.yoffset[i*2+0] = layout2.yoffset[i*2+1] = tempoffset[i];
		layout2.height *= 2;
	}
}

	Machine->uirotfont = decodegfx(fontdata_sb, &layout1);
	Machine->uirotfont2 = decodegfx(fontdata_db, &layout2);
	if (!Machine->uirotfont || !Machine->uirotfont2)
		return 0;

	Machine->uirotfont->colortable = colortable;
	Machine->uirotfont->total_colors  = MAX_COLORTABLE;

	Machine->uirotfont2->colortable = colortable;
	Machine->uirotfont2->total_colors = MAX_COLORTABLE;

	for (i = 0x4081; i < 0xfffe; i++)
		_uifonttable[i] = uifont_buildtable(i) + 256;

	return 1;
}


void uifont_freefont(void)
{
	if (Machine->uifont)
	{
		freegfx(Machine->uifont);
		Machine->uifont = NULL;
	}
	if (Machine->uirotfont)
	{
		freegfx(Machine->uirotfont);
		Machine->uirotfont = NULL;
	}
	if (Machine->uirotfont2)
	{
		freegfx(Machine->uirotfont2);
		Machine->uirotfont2 = NULL;
	}
}


int uifont_decodechar(unsigned char *s, unsigned short *code)
{
	unsigned char c1 = *(unsigned char *)s;
	unsigned char c2 = *(unsigned char *)(s + 1);
	int isDBC;

	switch (options.langcode)
	{
	default:
		isDBC = (c1 == 0x90) && (c2 > 0x80 && c2 < 0x80 + COLOR_BUTTONS);
	}

	if (isDBC)
	{
		*code = _uifonttable[(c1 << 8) | c2];

		if (*code == 0)
			*code = _uifonttable[0x817f];

		return 2;
	}

	*code = c1;
	return 1;
}

int uifont_drawfont(struct mame_bitmap *bitmap, const char *s, int sx,
		    int sy, int color)
{
	struct rectangle bounds;
	int x, y, wrapped, next_sy, increment, uifontwidth;
#ifdef UI_COLOR_DISPLAY
	int white = 0;
	int btn_offset, _code;
#endif /* UI_COLOR_DISPLAY */
	unsigned short code;
	unsigned char *c;
	struct GfxElement *uifont;

	c       = (unsigned char *)s;
	x       = sx;
	y       = sy;
	code    = 0;
	next_sy = 0;

#ifdef UI_COLOR_DISPLAY
	if (color == UI_COLOR_INVERSE)
	{
		white = Machine->uifont->colortable[FONT_COLOR_NORMAL];
		Machine->uifont->colortable[FONT_COLOR_NORMAL] = Machine->uifont->colortable[FONT_COLOR_SPECIAL];
	}
#endif /* UI_COLOR_DISPLAY */

	while (*c)
	{
		wrapped = 0;

		increment = uifont_decodechar(c, &code);

		if (*c == '\n')
		{
			x = sx;
			y += Machine->uifontheight + 1;
			next_sy += Machine->uifontheight;
			c++;
			continue;
		}
		else
		{
			if (code > 0x00ff && Machine->uirotfont2 != NULL)
			{
				uifont      = Machine->uirotfont2;
				uifontwidth = Machine->uifontwidth * 2;
			}
			else
			{
				uifont      = Machine->uirotfont;
				uifontwidth = Machine->uifontwidth;
			}
		}

		if (y >= Machine->uiheight)
			return next_sy;

		if (!wrapped)
		{
#ifdef UI_COLOR_DISPLAY
			pen_t col = 0;
#endif /* UI_COLOR_DISPLAY */

			if (code > 0x00ff)
			{
#ifdef UI_COLOR_DISPLAY
				static pen_t color_table[COLOR_BUTTONS] = {
					0,              		// dummy
					BUTTON_COLOR_RED,		// BTN_A
					BUTTON_COLOR_YELLOW,	// BTN_B
					BUTTON_COLOR_GREEN,	// BTN_C
					BUTTON_COLOR_BLUE,	// BTN_D
					BUTTON_COLOR_PINK,	// BTN_P
					BUTTON_COLOR_PURPLE,	// BTN_K
					BUTTON_COLOR_AQUA,	// BTN_S
					0,              		// BTN_+
					BUTTON_COLOR_RED,		// BTN_1
					BUTTON_COLOR_YELLOW,	// BTN_2
					BUTTON_COLOR_GREEN,	// BTN_3
					BUTTON_COLOR_BLUE,	// BTN_4
					BUTTON_COLOR_PINK,	// BTN_5
					BUTTON_COLOR_PURPLE,	// BTN_6
					BUTTON_COLOR_AQUA,	// BTN_7
					BUTTON_COLOR_SILVER,	// BTN_8
					BUTTON_COLOR_NAVY,	// BTN_9
					BUTTON_COLOR_LIME,	// BTN_10
					0,
					0,
					0,
					0,				// DIRECTION
					0,				// ARROWS
					0,				// ARE
					0,				// HERE
					0,
					0,
					0,
					BUTTON_COLOR_GREEN,	// LP
					BUTTON_COLOR_YELLOW,	// MP
					BUTTON_COLOR_RED,		// SP
					BUTTON_COLOR_GREEN,	// LK
					BUTTON_COLOR_YELLOW,    // MK
					BUTTON_COLOR_RED,		// SK
					BUTTON_COLOR_PURPLE,	// 3K
					BUTTON_COLOR_PINK,	// 3P
					BUTTON_COLOR_PURPLE,	// 2K
					BUTTON_COLOR_PINK,	// 2P
					0,				// TAP
					BUTTON_COLOR_GREEN,	// BTN_N
					BUTTON_COLOR_PURPLE,	// BTN_Z
					BUTTON_COLOR_RED,		// JOY BALL
				};

#endif /* UI_COLOR_DISPLAY */
				code -= 256;

#ifdef UI_COLOR_DISPLAY
				btn_offset = 0;
				_code = code;
								
				_code -= btn_offset;

				if (_code > 0 && _code < COLOR_BUTTONS)
					col = color_table[_code];

				if (col)
				{
					white = Machine->uifont->colortable[FONT_COLOR_NORMAL];
					Machine->uifont->colortable[FONT_COLOR_NORMAL] = Machine->uifont->colortable[col];
				}
#endif /* UI_COLOR_DISPLAY */
			}

			bounds.min_x = x + Machine->uixmin;
			bounds.min_y = y + Machine->uiymin;
			bounds.max_x = x + Machine->uixmin + uifontwidth - 1;
			bounds.max_y = y + Machine->uiymin + Machine->uifontheight - 1;
			ui_rot2raw_rect(&bounds);

			drawgfx(bitmap,
			        uifont,
			        code,
#ifdef UI_COLOR_DISPLAY
			        UI_COLOR_NORMAL,
#else /* UI_COLOR_DISPLAY */
				color,
#endif /* UI_COLOR_DISPLAY */
			        0,
			        0,
			        bounds.min_x,
			        bounds.min_y,
			        0,
#ifdef UI_COLOR_DISPLAY
			        TRANSPARENCY_PEN,
#else /* UI_COLOR_DISPLAY */
			        TRANSPARENCY_NONE,
#endif /* UI_COLOR_DISPLAY */
			        0);


			ui_markdirty(&bounds);

			x += uifontwidth;

			while (increment)
			{
				c++;
				increment--;
			}
			
#ifdef UI_COLOR_DISPLAY
			if (col)
				Machine->uifont->colortable[FONT_COLOR_NORMAL] = white;
#endif /* UI_COLOR_DISPLAY */
		}
	}

#ifdef UI_COLOR_DISPLAY
	if (color == UI_COLOR_INVERSE)
		Machine->uifont->colortable[FONT_COLOR_NORMAL] = white;
#endif /* UI_COLOR_DISPLAY */

	return next_sy;
}


#ifdef UI_COLOR_DISPLAY
void convert_command_move(char *buf)
{
	unsigned char *c = (unsigned char *)buf;
	int c1 = 0;

	switch (options.langcode)
	{
		default:
			c1 = 0x80;

			while (*c)
			{
				if (*c == '_')
				{
					*c++ = 0x90;
					switch (*c)
					{
						case 'A':
						case 'B':
						case 'C':
						case 'D':
							*c = *c - 'A' + 0x01 + c1;
							break;
						case 'E':
						case 'F':
						case 'G':
						case 'H':
						case 'I':
						case 'J':
							*c = *c - 'E' + 0x1d + c1;
							break;
						case 'K':
							*c = 0x06 + c1;
							break;
						case 'L':
						case 'M':
						case 'O':
						case 'Q':
						case 'R':
							*c = *c - 'L' + 0x4a + c1;
							break;
						case 'N':
							*c = 0x18 + c1;
							break;
						case 'P':
							*c = 0x05 + c1;
							break;
						case 'S':
							*c = 0x07 + c1;
							break;
						case 'T':
						case 'U':
						case 'V':
						case 'W':
						case 'X':
						case 'Y':
						case 'Z':
							*c = *c - 'T' + 0x23 + c1;
							break;
						case 'a':
						case 'b':
						case 'c':
						case 'd':
						case 'e':
						case 'f':
						case 'g':
						case 'h':
						case 'i':
						case 'j':
							*c = *c - 'a' + 0x09 + c1;
							break;
						case 'k':
						case 'l':
						case 'm':
						case 'n':
						case 'o':
						case 'p':
						case 'q':
						case 'r':
						case 's':
						case 't':
						case 'u':
						case 'v':
						case 'w':
						case 'x':
						case 'y':
						case 'z':
							*c = *c - 'k' + 0x3a + c1;
							break;
						case '0':
						case '1':
						case '2':
						case '3':
						case '4':
						case '6':
						case '7':
						case '8':
						case '9':
							*c = *c - '0' + 0x13 + c1;
							break;
						case '5':
							*c = 0x2a + c1;
							break;
						case '`':
							*c = 0x2b + c1;
							break;
						case '@':
							*c = 0x2c + c1;
							break;
						case '(':
							*c = 0x2d + c1;
							break;
						case ')':
							*c = 0x2e + c1;
							break;
						case '!':
							*c = 0x2f + c1;
							break;
						case '*':
							*c = 0x30 + c1;
							break;
						case '&':
							*c = 0x31 + c1;
							break;
						case '^':
							*c = 0x32 + c1;
							break;
						case '%':
							*c = 0x33 + c1;
							break;
						case '$':
							*c = 0x34 + c1;
							break;
						case '#':
							*c = 0x35 + c1;
							break;
						case '<':
							*c = 0x36 + c1;
							break;
						case '>':
							*c = 0x37 + c1;
							break;
						case '{':
							*c = 0x38 + c1;
							break;
						case '}':
							*c = 0x39 + c1;
							break;
						case '+':
							*c = 0x08 + c1;
							break;
						case '|':
							*c = 0x4c + c1;
							break;
						case '?':
							*c = 0x4e + c1;
							break;
						case '-':
							*c = 0x51 + c1;
							break;
						case '=':
							*c = 0x52 + c1;
							break;
						case ']':
							*c = 0x53 + c1;
							break;
						case '[':
							*c = 0x54 + c1;
							break;
						case '~':
							*c = 0x55 + c1;
							break;
					}
				}

				c++;
			}

			return;
	}


	while (*c)
	{
		unsigned short code;

		if (uifont_decodechar(c, &code) == 2)
		{
			c+=2;
		}
		else
		{
			if (*c == '_')
			{
				switch(*(c+1))
				{
					case 'A':
					case 'B':
					case 'C':
					case 'D':
						*c     = c1;
						*(c+1) = 0xc0 + (*(c+1) - 'A');
						c+=2;
						break;
					case 'P':
						*c     = c1;
						*(c+1) = 0xc4;
						c+=2;
						break;
					case 'K':
						*c     = c1;
						*(c+1) = 0xc5;
						c+=2;
						break;
					case 'S':
						*c     = c1;
						*(c+1) = 0xc6;
						c+=2;
						break;
					case '+':
						*c     = c1;
						*(c+1) = 0xc7;
						c+=2;
						break;
					case 'a':
					case 'b':
					case 'c':
					case 'd':
					case 'e':
					case 'f':
					case 'g':
					case 'h':
					case 'i':
					case 'j':
						*c     = c1;
						*(c+1) = 0xc8 + (*(c+1) - 'a');
						c+=2;
						break;
					case '1':
					case '2':
					case '3':
					case '4':
					case '6':
					case '7':
					case '8':
					case '9':
						*c     = c1;
						*(c+1) = 0xd2 + (*(c+1) - '0');
						c+=2;
						break;
					case 'N':
						*c     = c1;
						*(c+1) = 0xd7;
						c+=2;
						break;
					default:
						c++;
						break;
				}
			}
			else if (*c == '\\' && *(c+1) == 'n')
			{
				*c     = ' ';
				*(c+1) = '\n';
				c+=2;
			}
			else
				c++;
		}
	}
}
#endif /* UI_COLOR_DISPLAY */

